<?php

class Events_Api {
	protected $args;

	/**
	 * @var array
	 */
	protected $events = [];

	/**
	 * @var array
	 */
	protected $pagination = [];

	/**
	 * @var Performer_Api
	 */
	protected $performer_api;

	/**
	 * @var string
	 */
	protected $cache_key;

	public function __construct( $args ) {
		$this->args          = $args;
		$this->performer_api = new Performer_Api();
		$this->cache_key     = $this->generate_cache_key();
	}

	protected function maybe_get_cached_events() {
		$maybe_events = get_transient( $this->cache_key );
		if ( ! $maybe_events ) {
			return false;
		}

		return unserialize( $maybe_events );
	}

	protected function generate_cache_key() {
		$cache_args = $this->args;
		unset( $cache_args[ 'page_size' ] );
		unset( $cache_args[ 'page' ] );

		return 'eb_events_' . md5( wp_json_encode( $cache_args ) );
	}

	public function set_events( $events ) {
		$this->events = $events;
	}

	/**
	 * @return array
	 */
	public function get_events( $include_performers = false ) {
		if ( ! $include_performers ) {
			return $this->events;
		}

		foreach ( $this->events as $key => $event ) {
			$this->events[ $key ]->top_performances = $this->performer_api->get_top_performers( $event->id );
		}

		return $this->events;
	}

	/**
	 * @return $this
	 */
	public function request() {
		$maybe_events              = false; //$this->maybe_get_cached_events();
		$page_size                 = $this->args[ 'page_size' ] ? absint( $this->args[ 'page_size' ] ) : 200;
		$page                      = $this->args[ 'page' ] ? absint( $this->args[ 'page' ] ) : 1;
		$this->args[ 'page' ]      = 1;
		$this->args[ 'page_size' ] = 50; //max request for events, needs to be 50 or less to not break expansions

		if ( $maybe_events ) {
			$this->events = array_slice(
				$maybe_events,
				( $page_size * ( $page - 1 ) ),
				$page_size
			);

			$this->pagination = [
				'object_count' => count( $maybe_events ),
				'page_size'    => $page_size,
				'page_number'  => $page,
				'page_count'   => absint( ceil( count( $maybe_events ) / $page_size ) ),
			];

			return $this;
		}

		if ( ! isset( $this->args[ 'organizers' ] ) && ! isset( $this->args[ 'venues' ] ) ) {
			return $this; //We MUST have organizer or venue IDs to make requests.
		}

		$base_url = trailingslashit( EVB_API_ENDPOINT ) . 'v3/organizations/%s/events';

		//figure out what organization id's we need.
		if ( isset( $this->args[ 'organizers' ] ) ) {
			$ids = is_array( $this->args[ 'organizers' ] )
				? $this->args[ 'organizers' ]
				: explode( ',', $this->args[ 'organizers' ] );
			unset( $this->args[ 'organizers' ] );
		} else {
			$venue_org_map        = get_option( Oauth::VENUES_ORG_KEY, [] ); //get a map or venues by organization
			$venue_ids            = is_array( $this->args[ 'venues' ] ) ? $this->args[ 'venues' ] : [ $this->args[ 'venues' ] ]; // get our venue ids
			$ids                  = [];
			$subset_venue_org_map = []; //tracking organizations and venues used in the request, build org_ids and venue ids below.
			/**
			 * The reason for all of this is that the EB api doesn't have a good support for requesting events by venue,
			 * you must request events by organization with the venue filter. :(
			 */
			foreach ( $venue_ids as $venue_id ) {
				$org_id = $this->find_org_id_by_venue( $venue_id, $venue_org_map );
				if ( ! $org_id ) {
					continue;
				}
				$ids[]                             = $org_id;
				$subset_venue_org_map[ $org_id ][] = $venue_id;
			}
			$ids = array_unique( $ids );
			unset( $this->args[ 'venues' ] );
		}

		$this->args[ 'token' ] = get_option( 'api_access_token', '' );
		$events                = [];
		foreach ( $ids as $id ) {
			if ( isset( $subset_venue_org_map ) && ! empty( $subset_venue_org_map ) && isset( $subset_venue_org_map[ $id ] ) ) {
				$this->args[ 'venue_filter' ] = implode( ',', $subset_venue_org_map[ $id ] );
			}
			$url      = sprintf( $base_url, absint( $id ) ) . '?' . http_build_query( $this->args );
			$response = wp_remote_get( $url );
			file_put_contents( "/var/www/motorco/eb-debug.log", date('d F Y, h:i:s A') . " DEBUG EB: $url: ", FILE_APPEND | LOCK_EX);
			file_put_contents( "/var/www/motorco/eb-debug.log", "DEBUG EB: response: " . print_r($response, true), FILE_APPEND | LOCK_EX );
			echo "<!-- DEBUG EB " . $url . "-->";
			if ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
				echo "<!-- EB Response: " . $response . "-->";
				continue;
			}

			$body = json_decode( wp_remote_retrieve_body( $response ) );
			if ( empty( $body->events ) ) {
				continue;
			}

			//Merge our events
			$events = array_merge( $events, $body->events );
		}

		//Sort for standard start date order
		if ( $this->args[ 'order_by' ] === 'start_asc' ) {
			//We have enough events so let's stop the loop
			usort( $events, function ( $a, $b ) {
				return strtotime( $a->start->local ) - strtotime( $b->start->local );
			} );
		}

		//Sort event by just announced
		if ( $this->args[ 'order_by' ] === 'created_desc' ) {
			usort( $events, function ( $a, $b ) {
				return strtotime( $b->created ) - strtotime( $a->created );
			} );
		}

		$events = array_filter($events, function($event) {
			return $event->listed; //Don't show private events
		});

		set_transient( $this->cache_key, serialize( $events ), 3600 );

		$this->events = array_slice(
			$events,
			( $page_size * ( $page - 1 ) ),
			$page_size
		);

		$this->pagination = [
			'object_count' => count( $events ),
			'page_size'    => $page_size,
			'page_number'  => $page,
			'page_count'   => absint( ceil( count( $events ) / $page_size ) ),
		];

		return $this;
	}

	protected function find_org_id_by_venue( $venue_id, $venue_map ) {
		foreach ( $venue_map as $org_id => $venue_ids ) {
			if ( in_array( $venue_id, $venue_ids ) ) {
				return $org_id;
			}

			continue;
		}
	}

	public function get_pagination() {
		return $this->pagination;
	}

	/**
	 * @param $event_id
	 *
	 * @return array
	 */
	public function get_event( $event_id ) {
		$key         = sprintf( 'eb_event_cache_%d', $event_id );
		$maybe_event = get_transient( $key );
		if ( $maybe_event ) {
			return unserialize( $maybe_event );
		}
		$base_url = trailingslashit( EVB_API_ENDPOINT ) . 'v3/events/%d';
		$token    = get_option( 'api_access_token', '' );
		$url      = sprintf( $base_url, absint( $event_id ) ) . '?token=' . $token;
		$response = wp_remote_get( $url );
			file_put_contents( "/var/www/motorco/eb-debug.log", date('d F Y, h:i:s A') . " DEBUG EB: $url: ", FILE_APPEND | LOCK_EX);
			file_put_contents( "/var/www/motorco/eb-debug.log", "DEBUG EB: response: " . print_r($response, true), FILE_APPEND | LOCK_EX );

		if ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
			error_log("EB Error: url= $url response: ". print_r($response,true));
			return [];
		}
		$body = json_decode( wp_remote_retrieve_body( $response ) );
		set_transient( $key, serialize( $body ), 3600 );

		return $body;
	}

}
